#include "OgreSceneNodeHierarchyInterface.h"
#include <OgreIteratorWrappers.h>
#include <OgreCamera.h>
#include <OgreSceneNode.h>

//#include <windows.h>

namespace Ogre {

//-----------------------------------------------------------------------
SceneNodeHierarchyInterface::SceneNodeHierarchyInterface(SceneManager *sm, RenderSystem *rsys): 
PlatformHierarchyInterface(sm, rsys)
{		
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::TraverseNode(GtpVisibility::HierarchyNode *node)
{
	mNumTraversedNodes ++;
	
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	if (HasGeometry(node))
	{
		RenderNode(node);
	}

	// internal node: add children to priority queue for further processing
	Node::ChildNodeIterator it = sceneNode->getChildIterator();
				
	while (it.hasMoreElements())			
	{ 
		mDistanceQueue->push(it.getNext());
	}
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::RenderNode(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	if (sceneNode->lastRendered() != sceneNode->lastVisited())
	{
		sceneNode->setLastRendered(sceneNode->lastVisited());
		mNumRenderedNodes ++;

		mSceneManager->_renderSceneNode(mCamera, sceneNode);
	}
#endif
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::IsLeaf(GtpVisibility::HierarchyNode *node)
{
	return (static_cast<SceneNode *>(node)->numChildren() == 0);
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::HasGeometry(GtpVisibility::HierarchyNode *node)
{
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	
	return sceneNode->numAttachedObjects() > 0;
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::PullUpVisibility(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	while(sceneNode && !sceneNode->isNodeVisible())
	{
		sceneNode->setNodeVisible(true);
		sceneNode = sceneNode->getParentSceneNode();
	}
#endif
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::HasGreaterDistance(GtpVisibility::HierarchyNode *node1, 
													 GtpVisibility::HierarchyNode *node2)
{
	SceneNode *sceneNode1 = static_cast<SceneNode *>(node1);
	SceneNode *sceneNode2 = static_cast<SceneNode *>(node2);

	return sceneNode1->getSquaredViewDepth(mCamera) > sceneNode2->getSquaredViewDepth(mCamera);
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::SetNodeVisible(GtpVisibility::HierarchyNode *node, 
												 const bool visible)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	sceneNode->setNodeVisible(visible);
#endif
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::SetLastVisited(GtpVisibility::HierarchyNode *node, const int frameId)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	sceneNode->setLastVisited(frameId);
#endif
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::IsNodeVisible(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	return sceneNode->isNodeVisible();
#else
	return true;
#endif
}
//-----------------------------------------------------------------------
int SceneNodeHierarchyInterface::LastVisited(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	return sceneNode->lastVisited();
#else
	return 0;
#endif
}
//-----------------------------------------------------------------------
AxisAlignedBox *SceneNodeHierarchyInterface::GetBoundingBox(GtpVisibility::HierarchyNode *node)
{
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	return &mBox;
}
} // namespace Ogre	
